from enum import Enum


class Snoo(str, Enum):
    SNOO_ANGRY = 'snoo_angry'
    SNOO_FEELSBADMAN = 'snoo_feelsbadman'
    UPVOTE = 'upvote'
    SNOO_HEARTEYES = 'snoo_hearteyes'
    SNOO_FEELSGOODMAN = 'snoo_feelsgoodman'
    KAPPA = 'kappa'
    SNOO_TONGUE = 'snoo_tongue'
    CAT_BLEP = 'cat_blep'
    SNOO = 'snoo'
    SNOO_CRY = 'snoo_cry'
    SNOO_WINK = 'snoo_wink'
    SNOO_SAD = 'snoo_sad'
    REDDITGOLD = 'redditgold'
    SNOO_THOUGHTFUL = 'snoo_thoughtful'
    CAKE = 'cake'
    SNOO_SURPRISED = 'snoo_surprised'
    UPVOTE_PRIDE = 'upvote_pride'
    SNOO_JOY = 'snoo_joy'
    GIVE_UPVOTE_PRIDE = 'give_upvote_pride'
    SNOO_PUTBACK = 'snoo_putback'
    SNOO_FACEPALM = 'snoo_facepalm'
    SNOO_SIMPLE_SMILE = 'snoo_simple_smile'
    SNOO_SCREAM = 'snoo_scream'
    SNOO_SMILE = 'snoo_smile'
    ILLUMINATI = 'illuminati'
    PUPPER = 'pupper'
    SNOO_SHRUG = 'snoo_shrug'
    DOWNVOTE = 'downvote'
    SLOTH = 'sloth'
    SNOO_HUG = 'snoo_hug'
    SNOO_TROLLFACE = 'snoo_trollface'
    HEART_EYES_PRIDE = 'heart_eyes_pride'
    DOGE = 'doge'
    SNOO_DEALWITHIT = 'snoo_dealwithit'
    SNOO_DISAPPROVAL = 'snoo_disapproval'
    PARTYPARROT = 'partyparrot'
    SNOO_TABLEFLIP = 'snoo_tableflip'
    KARMA = 'karma'
    ORLY = 'orly'
    HAMSTER = 'hamster'


class Reaction(str, Enum):
    REACT1 = 'mi2jolzfa7081.gif'
    REACT2 = '2o3aooqfa7081.gif'
    REACT3 = 'k7ry7t1ga7081.gif'
    REACT4 = 'jvuspmbga7081.gif'
    REACT5 = 'g6akcitga7081.gif'
    REACT6 = '3i2arwzga7081.gif'
    REACT7 = 'cdnhvqfga7081.gif'
    REACT8 = 't5bqwxdyt7081.gif'
    REACT9 = 'tspuf53ga7081.gif'
    REACT10 = 'ax7wu47ga7081.gif'
    REACT11 = 't2r5xc9ga7081.gif'
    REACT12 = '0ku6twega7081.gif'
    REACT13 = 'b5s6cohga7081.gif'
    REACT14 = 'pleyoikga7081.gif'
    REACT15 = 'zn7iubvfa7081.gif'
    REACT16 = '19b5q4vga7081.gif'
    REACT17 = '8r21ukpfa7081.gif'
    REACT18 = 'm7uy86lga7081.gif'
    REACT19 = 'wbrgz1nga7081.gif'
    REACT20 = 'vq7naqwfa7081.gif'
    REACT21 = 'qzl5vyxfa7081.gif'
    REACT22 = 'sjs1a2fyt7081.gif'
    REACT23 = '9ut7iedga7081.gif'
    REACT24 = '3s0glewga7081.gif'
    REACT25 = '00brcfjga7081.gif'
    REACT26 = 'dfxygs4ga7081.gif'
    REACT27 = '1p1jgpgga7081.gif'
    REACT28 = 't1djdguga7081.gif'
    REACT29 = '79opsq0ha7081.gif'
    REACT30 = 'foyijyyga7081.gif'
    REACT31 = 'uy83aa8ga7081.gif'
    REACT32 = 'mag7v6tfa7081.gif'
    REACT33 = 'xjls1pqga7081.gif'
    REACT34 = 'ksz4fmaga7081.gif'
    REACT35 = 'evwks24ga7081.gif'
    REACT36 = 'd45pfmsga7081.gif'
    REACT37 = 'iuqmp7ufa7081.gif'
    REACT38 = '8zx5ixoga7081.gif'
    REACT39 = 'ytv3x0sfa7081.png'
    REACT40 = '4zhlw4iyt7081.gif'
    REACT41 = 'av9z8iiga7081.gif'
    REACT42 = '7rc03trga7081.gif'
    REACT43 = 'mp9zclcga7081.gif'
    REACT44 = 'fsg2a1oga7081.gif'
    REACT45 = '7url39xga7081.gif'
    REACT46 = 'd2kn6yxga7081.gif'
    REACT47 = '8kw138jyt7081.gif'
    REACT48 = 'ul2w17ega7081.gif'
